@@ -7,6 +7,7 @@ from presto import __version__, __date__
 
 # Imports
 import math
+import re
 import os
 import sys
 from collections import OrderedDict
@@ -18,19 +19,24 @@ from presto.Defaults import default_delimiter, default_barcode_field
 from presto.Annotation import parseAnnotation
 
 
-def readPrimerFile(primer_file):
+def readPrimerFile(primer_file, replace_special=True):
     """
     Processes primer sequences from file
 
     Arguments:
       primer_file (str): name of the FASTA file containing primer sequences.
+      replace_special (bool): if True replace whitespace and pRESTO annotation
+                              delimiters with an underscore.
 
     Returns:
       dict: Dictionary mapping primer ID to sequence.
     """
+    if replace_special:  parse_id = lambda x: re.sub('[\s,=|]+', '_', x)
+    else:  parse_id = lambda x: x
+
     with open(primer_file, 'r') as primer_handle:
         primer_iter = SeqIO.parse(primer_handle, 'fasta')
-        primers = {p.description: str(p.seq).upper() for p in primer_iter}
+        primers = {parse_id(p.description): str(p.seq).upper() for p in primer_iter}
 
     return primers
 
@@ -375,37 +381,43 @@ def printProgress(current, total, step, start_time=None, task=None, end=False):
         sys.stdout.flush()
 
 
-def printWarning(message):
+def printWarning(message, newline=False):
     """
     Prints a warning to standard error
 
     Arguments:
       message (str): warning message.
+      newline (bool): prefix with a newline if True.
     """
-    sys.stderr.write('WARNING> %s\n' % message)
+    prefix = '\n' if newline else ''
+    sys.stderr.write('%sWARNING> %s\n' % (prefix, message))
 
 
-def printError(message, exit=True):
+def printError(message, exit=True, newline=False):
     """
     Prints an error to standard error and exits
 
     Arguments:
       message (str): error message.
       exit (bool): if True exit after the message.
+      newline (bool): prefix with a newline if True.
     """
+    prefix = '\n' if newline else ''
     if exit:
-        sys.exit('ERROR> %s\n' % message)
+        sys.exit('%sERROR> %s\n' % (prefix, message))
     else:
-        sys.stderr.write('ERROR> %s\n' % message)
+        sys.stderr.write('%sERROR> %s\n' % (prefix, message))
 
 
-def printDebug(message, debug=True):
+def printDebug(message, debug=True, newline=False):
     """
     Prints a debug message to standard error
 
     Arguments:
       message (str): message.
       debug (bool): if True print the message.
+      newline (bool): prefix with a newline if True.
     """
     if debug:
-        sys.stderr.write('DEBUG> %s\n' % message)
\ No newline at end of file
+        prefix = '\n' if newline else ''
+        sys.stderr.write('%sDEBUG> %s\n' % (prefix, message))
