@@ -11,6 +11,7 @@ from presto import __version__, __date__, default_delimiter
 import numpy as np
 import pandas as pd
 import re
+import math
 from collections import OrderedDict
 from itertools import product, zip_longest, groupby
 from scipy import stats as stats
@@ -35,6 +36,9 @@ from presto.Applications import runBlastn, runUBlast
 from presto.IO import printWarning, printError
 from presto.Multiprocessing import SeqResult
 
+# Constants
+qual_to_prob = tuple(10 ** (-qual / 10) for qual in range(128))
+
 
 class PrimerAlignment:
     """
@@ -178,7 +182,7 @@ class AssemblyStats:
 
 def translateAmbigDNA(key):
     """
-    Translates IUPAC Ambiguous Nucleotide characters to or from character sets
+    Translates IUPAC ambiguous nucleotide characters to or from character sets
 
     Arguments:
       key : String or re.search object containing the character set to translate
@@ -188,10 +192,10 @@ def translateAmbigDNA(key):
     """
     # Define valid characters and character translations
     IUPAC_uniq = '-.ACGT'
-    IUPAC_ambig = 'BDHKMNRSVWY'
+    IUPAC_ambig = 'UBDHKMNRSVWY'
     IUPAC_trans = {'AG':'R', 'CT':'Y', 'CG':'S', 'AT':'W', 'GT':'K', 'AC':'M',
-                  'CGT':'B', 'AGT':'D', 'ACT':'H', 'ACG':'V', 'ABCDGHKMRSTVWY':'N',
-                  '-.':'.'}
+                   'CGT':'B', 'AGT':'D', 'ACT':'H', 'ACG':'V', 'ACGTUBDHKMRSVWY':'N',
+                   'T':'U', '-.':'.'}
 
     # Convert passed regular expression match to a string
     if hasattr(key, 'group'): key = key.group(1)
@@ -214,10 +218,10 @@ def translateAmbigDNA(key):
 
 def scoreDNA(a, b, mask_score=None, gap_score=None):
     """
-    Returns the score for a pair of IUPAC Ambiguous Nucleotide characters
+    Returns the score for a pair of IUPAC ambiguous nucleotide characters
 
     Arguments:
-      a : First characters
+      a : First character
       b : Second character
       n_score : Tuple of length two defining scores for all matches against an N
                 character for (a, b), with the score for character (a) taking precedence;
@@ -230,9 +234,9 @@ def scoreDNA(a, b, mask_score=None, gap_score=None):
       int : Score for the character pair
     """
     # Define ambiguous character translations
-    IUPAC_trans = {'AGWSKMBDHV':'R', 'CTSWKMBDHV':'Y', 'CGKMBDHV':'S', 'ATKMBDHV':'W', 'GTBDHV':'K',
-                   'ACBDHV':'M', 'CGTDHV':'B', 'AGTHV':'D', 'ACTV':'H', 'ACG':'V', 'ABCDGHKMRSTVWY':'N',
-                   '-.':'.'}
+    IUPAC_trans = {'AGWSKMBDHV':'R', 'CTUSWKMBDHV':'Y', 'CGKMBDHV':'S', 'ATUKMBDHV':'W', 'GTUBDHV':'K',
+                   'ACBDHV':'M', 'CGTUDHV':'B', 'AGTUHV':'D', 'ACTUV':'H', 'ACG':'V', 'ACGTUBDHKMRSVWY':'N',
+                   'T':'U', '-.':'.'}
 
     # Create list of tuples of synonymous character pairs
     IUPAC_matches = [p for k, v in IUPAC_trans.items() for p in list(product(k, v))]
@@ -317,7 +321,7 @@ def getDNAScoreDict(mask_score=None, gap_score=None):
     Returns:
       dict : Score dictionary with keys (char1, char2) mapping to scores
     """
-    chars = '-.ACGTRYSWKMBDHVN'
+    chars = '-.ACGTURYSWKMBDHVN'
     score_dict = {k:scoreDNA(*k, mask_score=mask_score, gap_score=gap_score)
                   for k in product(chars, repeat=2)}
 
@@ -1258,6 +1262,27 @@ def filterRepeats(data, max_repeat=default_filter_max_repeat, include_missing=Fa
 
     return result
 
+def meanQuality(qual, prob=qual_to_prob):
+    """
+    Calculate mean quality score
+
+    Arguments:
+      qual (list): numeric Phred quality scores.
+      prob (list): mapping of Phred score (index) to probability values
+
+    Returns:
+      int: floor of the mean Phred quality score.
+    """
+    # Assign 0 to empty list
+    if len(qual) == 0:
+        return 0
+
+    qual_sum = 0.0
+    for q in qual:  qual_sum += prob[q]
+    p = qual_sum / len(qual)
+
+    return math.floor(-10 * math.log10(p))
+
 
 def filterQuality(data, min_qual=default_consensus_min_qual, inner=True,
                   missing_chars=''.join(default_missing_chars)):
@@ -1284,13 +1309,11 @@ def filterQuality(data, min_qual=default_consensus_min_qual, inner=True,
     else:
         quals = seq.letter_annotations['phred_quality']
 
-    if len(quals) > 0:
-        q = sum(quals) / len(quals)
-    else:
-        q = 0
+    # Calculate mean quality
+    mean_qual = meanQuality(quals)
 
     # Build result object
-    valid = (q >= min_qual)
+    valid = (mean_qual >= min_qual)
     result = SeqResult(data.id, seq)
     if valid:
         result.results = seq
@@ -1298,7 +1321,7 @@ def filterQuality(data, min_qual=default_consensus_min_qual, inner=True,
 
     # Update result log
     result.log['SEQ'] = seq.seq
-    result.log['QUALITY'] = q
+    result.log['QUALITY'] = mean_qual
 
     return result
 
